'２次元構造解析V116の標準ﾓｼﾞｭｰﾙ        ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module FEM2Com
    '平面応力、平面歪、板曲げ、軸対称体の弾性解析
    'アイソメトリック四角形要素使用
    '節点応力も計算可能
    '自動メッシュ生成は「ＦＥＭ断面データ」ソフトによる
    '要素の節点番号の入力は、反時計回り
    '表面応力等の節点番号入力は、物体を左側に置く

    'V106での改良点・・・板の曲げで平面分布荷重の換算設定を追加
    'V107での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V108での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V109での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                    （これまでは最長方向の座標軸を採用）
    'V110での改良点・・・構造図画面で条件設定部の表示を２区分から３区分に改善、
    '                    構造図画面で条件設定部の表示辺幅を拡大
    'V111での改良点・・・拘束条件で座標軸方向の欠けエラーをチェック
    'V112での改良点・・・固定境界の一括設定を追加
    'V113での改良点・・・入力データファイルで拘束節点の形式を改善
    'V114での改良点・・・サブメニューの中間節点自動計算を廃止
    'V115での改良点・・・解析データのデータチェックに要素共通辺の節点を追加
    'V116での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$                 '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$                 '立ち上げ時のディレクトリー名
    Public Fadd$                    '連結するファイル名
    Public Fm$                      '入力データのファイル名
    Public Fy$                      '計算結果のファイル名
    Public Title$                   'タイトル
    Public VarName$                 '対象変数名

    Public Timst As Date            '計算開始時刻

    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力、2:Therm2Dﾌｧｲﾙ入力）
    Public Kaxis As Integer         '作図のＸＹ軸の入換え（0:なし、1:有り）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '要素辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KDrawDatMod As Integer   '構造図でのﾃﾞｰﾀ修正区分（0:個別、1:一括）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public Kfile As Integer         '出力ファイルのポート番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindDPL As Integer       '変形区分（0:小変形、1:大変形）
    Public KindGrav As Integer      '自重計算の有無（0:なし、1:有り）
    Public KindPost As Integer      '後処理の種類（1:等高線図、2:変位図、3:端数処理、
    '                                4:主要値表示）
    Public KindRevF As Integer      '回転遠心力の有無（0:なし、1:有り）
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:材料番号、2:拘束節点、3:節点荷重、
    '                                4:表面分布荷重、5:ばね支持節点、6:傾斜拘束節点）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileB As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public KStage As Integer        'ﾃﾞｰﾀ入力の段階番号
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public Kversion As Integer      '結果ファイルのバージョン番号（1～2）
    Public Mprop As Integer         '材料データの成分数(3～4)
    Public Nall As Integer          '変数の総数
    Public Nbound, Nboundb As Integer   '拘束節点数
    Public Ndel As Integer          '削除要素数
    Public Ndim As Integer          '１節点当りの変数の数(2～3)
    Public NDisp As Integer         '等高線の分割数
    Public Nelem As Integer         '要素数
    Public Nforce, Nforceb As Integer   '荷重節点数
    Public NforceA As Integer       '座標位置荷重数
    Public Ngaus As Integer         'ガウス積分点数(3)
    Public NKmax As Integer         '最大繰り返し計算回数
    Public Nmat As Integer          '材料データ数
    Public Npoint As Integer        '節点数
    Public Nslope, Nslopeb As Integer   '傾斜拘束節点数
    Public Nspring, Nspringb As Integer 'バネ支持個数
    Public Nsurf, Nsurfb As Integer     '表面分布荷重数
    Public Nstress As Integer       '応力成分の数
    Public Ntype As Integer         '問題のタイプ（1:平面応力、2:平面歪、3:板の曲げ、4:軸対称）
    Public Nvar As Integer          '１要素当りの変数の総数

    Public AccGrav As Single        '重力加速度
    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DispRatio As Single      '変位作画の倍率
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public EpsCalc As Single        '収束基準誤差率
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public RevPS As Single          '回転数（単位時間当り）
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'ﾓﾃﾞﾙのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'ﾓﾃﾞﾙの垂直軸上向き回りの回転角（右ねじ正、°）

    Public ArrText As New ArrayList()

    Public UFileB(6) As String      '最近使用した解析ﾌｧｲﾙの保存配列

    Public Jdirect(,) As Integer    '節点拘束の各軸方向拘束有無(0:非拘束、1:拘束）
    Public KAppPoint() As Integer   'ﾃﾞｰﾀ設定の節点
    Public KAppSide(,) As Integer   'ﾃﾞｰﾀ設定の要素指定辺
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KdrawElemOld() As Integer    '変更前の各要素の作画有無（0:なし、1:有り）
    Public KEInside(,) As Integer   '要素の各辺の内部区分（0:外周、1:内部）
    Public Koption(3) As Integer    'オプション有無（1:バックアップ、
    '                                2:要素入力画面の節点番号表示、3:節点入力画面の表示）
    Public KPforce() As Integer     '節点荷重の節点番号
    Public KPsurf(,) As Integer     '表面分布荷重の３節点番号または要素番号
    Public KPspring(,) As Integer   'バネ支持の節点番号
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '拘束の節点番号
    Public Nsrest() As Integer      '傾斜拘束の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号

    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public CenterPos(3) As Single   '構造中心の座標
    Public Cspring(,) As Single     'バネ定数
    Public DataMat(,) As Single     '各材料の物性値
    Public Ds(8), Dt(8) As Double   '形状関数のs偏微分、t偏微分
    Public Dx(8), Dy(8) As Double   '形状関数のx偏微分、y偏微分
    Public Force(,) As Single       '節点荷重値（各軸方向）
    Public ForceA(,) As Single      '座標位置荷重値（各軸方向）
    Public Fspring(,) As Single     '初期バネ力（各軸方向）
    Public GravMulti(2) As Single   '自重の各座標軸方向の倍率
    Public Pnorm() As Single        '表面の垂直方向分布荷重（単位面積当り）
    Public Ptang() As Single        '表面の接線方向分布荷重（単位面積当り）
    Public PosGaus(3) As Single     'ｶﾞｳｽ積分点の位置(於要素座標）
    Public SAngle() As Single       '傾斜拘束角度（°）
    Public Shape(8) As Double       '形状関数
    Public Tdisp(,) As Single       '結果の変位
    Public Tr(8, 9) As Single       'ｶﾞｳｽ点・節点の変換ﾏﾄﾘｯｸｽ
    Public TransMat(4, 4) As Single 'モデルの合成アフィン変換行列
    Public Tu() As Single           'FEM計算の解
    Public Vdirect(,) As Single     '節点拘束の各軸方向拘束値
    Public WeightG(3) As Single     'ｶﾞｳｽ積分点の重み
    Public X(), Y() As Single       '節点座標
    Public Xdraw(8), Ydraw(8) As Single     '色塗り多角形の座標
    Public XE(8), YE(8) As Single   '要素計算の配列
    Public XFA(), YFA() As Single    '座標位置荷重のX,Y座標
    Public Xmodel(), Ymodel(), Zmodel() As Single 'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xnew(), Ynew() As Single   '変位入りの節点座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xsect(), Ysect(), Zsect() As Single  '部分断面の位置座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）

    Sub AdjustRatio(ByRef ratio As Single)
        '変位比率の調整
        Dim nb As Integer
        Dim b1, b2, std As Single

        std = 0.05
        b1 = std / ratio
        If b1 < 1.0 Then
            ratio = 1.0
        ElseIf b1 < 10.0 Then
            b2 = b1 * 10
            ratio = Int(b2) / 10.0
        Else    '上位2桁を採用
            b2 = b1
            nb = 0
            Do
                b2 = b2 / 10
                nb = nb + 1
                If b2 < 10 Then
                    Exit Do
                End If
            Loop
            b2 = Int(b2 * 10)
            If nb < 2 Then
                ratio = b2
            Else
                ratio = b2 * 10 ^ (nb - 1)
            End If
        End If

    End Sub

    Sub AreaCalc(ByRef area As Single, ByVal x1 As Single, _
        ByVal y1 As Single, ByVal x2 As Single, ByVal y2 As Single, _
        ByVal xb As Single, ByVal yb As Single)
        '三角形の符号付き面積算出（最初の2点に対して左側は正、右側は負）
        'area･･･符号付き面積、三角形の2倍
        'x1,y1,x2,y2･･･2点の座標
        'xb,yb･･･対象点の座標
        area = x1 * y2 - y1 * x2 + x2 * yb - y2 * xb + xb * y1 - yb * x1

    End Sub

    Sub AxisCheck(ByRef icheck As Integer, ByRef k1 As Integer, _
    ByRef k2 As Integer, ByRef ratio As Single, ByRef xmin As Single, _
    ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single, _
    ByRef xd() As Single, ByRef yd() As Single)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        '         最初に-1が設定された場合は、RZ座標のR最小値は0としない
        'k1,k2･･･要素の開始、終了番号
        'ratio･･･作画直角回転なしの縦横比上限（指定値）
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        'xd(),yd()･･･節点の座標
        Dim i, ie, ik, j As Integer
        Dim b1, b2, ratiob As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = xd(i) : xmax = xmin
                        ymin = yd(i) : ymax = ymin
                    Else
                        If xd(i) < xmin Then
                            xmin = xd(i)
                        ElseIf xd(i) > xmax Then
                            xmax = xd(i)
                        End If
                        If yd(i) < ymin Then
                            ymin = yd(i)
                        ElseIf yd(i) > ymax Then
                            ymax = yd(i)
                        End If
                    End If
                Next j
            End If
        Next ie
        If Ntype = 4 And icheck >= 0 Then
            xmin = 0
        End If
        b1 = xmax - xmin
        b2 = ymax - ymin
        If b1 = 0 Then
            icheck = 0
        Else
            ratiob = b2 / b1
            If ratiob > ratio Then
                icheck = 1
            Else
                icheck = 0
            End If
        End If

    End Sub

    Sub AxisCheck2(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single, ByRef xd() As Single, ByRef yd() As Single)
        'X、Y軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        'xd(),yd(),zd()･･･節点の座標
        Dim ie, ii, ik, j, k As Integer
        Dim b1 As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = xd(k)
                        AxisMin(2) = yd(k)
                        For ii = 1 To 2
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 2
                            Select Case ii
                                Case 1
                                    b1 = xd(k)
                                Case 2
                                    b1 = yd(k)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = AxisMax(1) - AxisMin(1)
        b1 = AxisMax(2) - AxisMin(2)
        If b1 > rlenmax Then
            rlenmax = b1
        End If

    End Sub

    Sub AxisCheckS(ByVal k1 As Integer, ByRef k2 As Integer, ByRef xmin As Single, _
    ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single, _
    ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        'xdb(),ydb()･･･節点座標
        Dim i, ie, ik, j As Integer
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = xdb(i) : xmax = xmin
                        ymin = ydb(i) : ymax = ymin
                    Else
                        If xdb(i) < xmin Then
                            xmin = xdb(i)
                        ElseIf xdb(i) > xmax Then
                            xmax = xdb(i)
                        End If
                        If ydb(i) < ymin Then
                            ymin = ydb(i)
                        ElseIf ydb(i) > ymax Then
                            ymax = ydb(i)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisSet(ByRef tdb(,) As Single)
        '座標の設定
        Dim i As Integer
        For i = 1 To Npoint
            Xnew(i) = X(i) + DispRatio * tdb(i, 1)
            Ynew(i) = Y(i) + DispRatio * tdb(i, 2)
        Next i
    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf()･･･節点の座標
        Dim i, ie, kind, nb As Integer
        Dim b1, b2, pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa, X, Y)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        b1 = pa - XaxisMin
        b2 = qa - YaxisMin
        If b2 > b1 Then
            b1 = b2
        End If
        b2 = b1 * 0.00002
        Call RoundPrep(kind, nb, b2)    '丸め準備

        For i = 1 To Npoint
            xbuf(i) = X(i)
            ybuf(i) = Y(i)
        Next i
        For i = 1 To Npoint
            If xbuf(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If ybuf(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'kcbuf()･･･元の節点の変数順序
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        Dim i, ie, ii, j, jj, k, k1, k2, mx As Integer

        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To 7
                i = NNelem(ie, ii)
                For jj = ii + 1 To 8
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        If Nspring > 0 Then
            For j = 1 To Nspring
                k1 = KPspring(j, 1)
                k2 = KPspring(j, 2)
                If k2 > 0 Then
                    k = Math.Abs(kcbuf(k1) - kcbuf(k2))
                    If k > mx Then mx = k
                End If
            Next j
        End If
        mb = Ndim * (mx + 1)

    End Sub

    Sub BoundDupCheck(ByRef nd As Integer)
        '拘束節点重複のﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nbound) As Integer

        nd = 0
        For i = 1 To Nbound - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nbound
                    If Nrest(i) = Nrest(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Ndim
                            If Jdirect(ii, j) > 0 Then
                                Jdirect(i, j) = Jdirect(ii, j)
                                Vdirect(i, j) = Vdirect(ii, j)
                            End If
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nobuf(Nbound), jfbuf(Nbound, Ndim) As Integer
            Dim vfbuf(Nbound, Ndim) As Single
            For i = 1 To Nbound
                nobuf(i) = Nrest(i)
                For j = 1 To Ndim
                    jfbuf(i, j) = Jdirect(i, j)
                    vfbuf(i, j) = Vdirect(i, j)
                Next j
            Next i
            nb = Nbound - nd
            ReDim Nrest(nb), Jdirect(nb, Ndim), Vdirect(nb, Ndim)
            nb = 0
            For i = 1 To Nbound
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    Nrest(nb) = nobuf(i)
                    For j = 1 To Ndim
                        Jdirect(nb, j) = jfbuf(i, j)
                        Vdirect(nb, j) = vfbuf(i, j)
                    Next j
                End If
            Next i
            Nbound = nb
            Nboundb = nb
        End If

    End Sub

    Sub CenterLine(ByVal gr As Graphics, ByVal icheck As Integer)
        '中心線の作画
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        Dim myPen As New Pen(Color.Black)
        myPen.DashStyle = Drawing2D.DashStyle.DashDot
        Select Case icheck
            Case 0
                gr.DrawLine(myPen, Xlower, Ylower, Xlower, Yupper)
            Case 1
                gr.DrawLine(myPen, Xlower, Ylower, Xupper, Ylower)
        End Select

    End Sub

    Sub CfileDispU(ByVal fxx$, ByRef tfbuf(,) As Single)
        '計算結果ﾌｧｲﾙから変位ﾃﾞｰﾀの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        'tfbuf(,)･･･節点の各座標方向の変位
        Dim aa$
        Dim ip, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Do
            aa$ = sr.ReadLine
        Loop Until aa$ = "＊＊ 変位等の出力 ＊＊"
        aa$ = sr.ReadLine
        For ip = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To 2
                tfbuf(ip, j) = dat(j)
            Next j
        Next ip
        sr.Close()

    End Sub

    Sub CfileDispSU(ByVal fxx$, ByRef tfbuf(,) As Single)
        '計算結果ﾌｧｲﾙから変位ﾃﾞｰﾀの読み込み（板の曲げ）
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        'tfbuf(,)･･･節点の各座標方向の変位
        Dim aa$
        Dim ip As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Do
            aa$ = sr.ReadLine
        Loop Until aa$ = "＊＊ 変位等の出力 ＊＊"
        aa$ = sr.ReadLine
        For ip = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            tfbuf(ip, 1) = dat(1)
        Next ip
        sr.Close()

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim a1$, a2$, aa$, aerr1$
        Dim ib, ic, id, ie, im, ip, j, jb, jx, k, kb, kh As Integer
        Dim n1, n2, n3, nb, nx As Integer
        Dim b1, b2 As Single
        Dim abuf$(10)

        a2$ = ""
        aerr1$ = "になっています！"
        num = 0

        If Ntype < 1 And Ntype > 4 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  問題のタイプの番号が間違っています！"
        End If
        If Npoint < 8 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Nbound < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  拘束節点数が 0" + aerr1$
        End If
        If Nmat < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  材料データ数が" + Str$(Nmat) + aerr1$
        End If

        If KindRevF > 0 Then    '遠心回転力のﾁｪｯｸ
            If RevPS <= 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  回転数が" + Str$(RevPS) + aerr1$
            End If
            If AccGrav = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  重力加速度が" + Str$(AccGrav) + aerr1$
            End If
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Or nb > Nmat Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf$(num) = aa$ + " 材料番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = 1 To 8
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "の"
                    aa$ = aa$ + " 節点番号" + Str$(nb)
                    abuf$(num) = aa$ + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If j < 8 Then
                    For k = j + 1 To 8
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                            abuf$(num) = aa$ + " ２つの節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = X(ip) - X(j)
                b2 = Y(ip) - Y(j)
                If b1 = 0 And b2 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "が"
                    abuf$(num) = aa$ + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '不要節点のﾁｪｯｸ
        Dim kbuf(Npoint) As Integer
        For ie = 1 To Nelem
            For j = 1 To 8
                kbuf(NNelem(ie, j)) = 1
            Next j
        Next ie
        For ip = 1 To Npoint
            If kbuf(ip) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が使われていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ip

        '要素共通辺のﾁｪｯｸ
        ReDim kbuf(Npoint)
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                ib = NNelem(ie, j)
                If kbuf(ib) = 0 Then
                    kbuf(ib) = 1    '頂点節点に指定
                End If
            Next j
        Next ie
        For ie = 1 To Nelem
            For j = 2 To 8 Step 2
                ib = NNelem(ie, j)
                If kbuf(ib) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ib) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ie

        If Nbound > 0 Then     '拘束節点ﾃﾞｰﾀのﾁｪｯｸ
            Dim kdir(3) As Integer
            For ib = 1 To Nbound
                nb = Nrest(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  拘束番号" + Str$(ib) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                ic = 0
                For id = 1 To Ndim
                    If Jdirect(ib, id) <> 0 Then
                        ic = 1
                        kdir(id) = 1
                    End If
                Next id
                If ic = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  拘束番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 拘束がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
            ic = 1
            Select Case Ntype
                Case 3
                    If kdir(1) = 0 Then
                        ic = 0
                        a2$ = "Z方向が拘束されていません！"
                    End If
                Case 4
                    If kdir(2) = 0 Then
                        ic = 0
                        a2$ = "Z方向が拘束されていません！"
                    End If
                Case Else
                    If kdir(1) = 0 Or kdir(2) = 0 Then
                        ic = 0
                        a2$ = ""
                        If kdir(1) = 0 Then
                            a2$ = "X方向が拘束されていません！"
                        End If
                        If kdir(2) = 0 Then
                            If a2$ = "" Then
                                a2$ = "Y方向が拘束されていません！"
                            Else
                                a2$ = "X,Yの両方向が拘束されていません！"
                            End If
                        End If
                    End If
            End Select
            If ic = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  " + a2$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        End If

        '材料ﾃﾞｰﾀのﾁｪｯｸ
        For im = 1 To Nmat
            For j = 1 To Mprop
                b1 = DataMat(im, j)
                If b1 <= 0 And j <> 2 Then
                    If Ntype = 4 Then
                        If j = 3 Then
                            If KindGrav = 0 And KindRevF = 0 Then
                                Exit For
                            End If
                        End If
                    Else
                        If j = 4 Then
                            If KindGrav = 0 Then
                                Exit For
                            End If
                        End If
                    End If
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                    a1$ = Str$(b1) + aerr1$
                    Select Case j
                        Case 1
                            a2$ = " ヤング率が"
                        Case 3
                            If Ntype = 4 Then
                                a2$ = " 比重量が"
                            Else
                                a2$ = " 厚さが"
                            End If
                        Case 4
                            a2$ = " 比重量が"
                    End Select
                    abuf$(num) = aa$ + a2$ + a1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next im

        If Nforce > 0 Then  '荷重節点のﾁｪｯｸ
            For ib = 1 To Nforce
                nb = KPforce(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  荷重節点の荷重番号" + Str$(ib) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                ic = 0
                For id = 1 To Ndim
                    If Force(ib, id) <> 0 Then
                        ic = 1
                        Exit For
                    End If
                Next id
                If ic = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  荷重節点の荷重番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 荷重がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If NforceA > 0 Then  '座標位置荷重のﾁｪｯｸ
            For ib = 1 To NforceA
                ic = 0
                For id = 1 To Ndim
                    If ForceA(ib, id) <> 0 Then
                        ic = 1
                        Exit For
                    End If
                Next id
                If ic = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  座標位置荷重の荷重番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 荷重がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nsurf > 0 Then   '表面分布荷重のﾁｪｯｸ
            If Ntype = 3 Then
                For ib = 1 To Nsurf
                    jb = KPsurf(ib, 1)
                    If jb < 1 Or jb > Nelem Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "の"
                        abuf$(num) = aa$ + " 要素番号" + Str$(jb) + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    If Pnorm(ib) = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 荷重がありません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            Else
                Dim i1, i2, ii, j1, j2, jj, js As Integer
                Dim kmulti(Nelem, 8) As Integer
                For ie = 1 To Nelem
                    i1 = NNelem(ie, 8)
                    For j = 1 To 8
                        i2 = NNelem(ie, j)
                        For ii = ie + 1 To Nelem
                            j1 = NNelem(ii, 8)
                            For jj = 1 To 8
                                j2 = NNelem(ii, jj)
                                Call NumCheck(ic, i1, i2, j1, j2)   '要素辺の重なりﾁｪｯｸ
                                If ic = 1 Then
                                    kmulti(ie, j) = 1
                                    kmulti(ii, jj) = 1
                                End If
                                j1 = j2
                            Next jj
                        Next ii
                        i1 = i2
                    Next j
                Next ie
                For ib = 1 To Nsurf
                    For j = 1 To 3
                        jb = KPsurf(ib, j)
                        If jb < 1 Or jb > Npoint Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "の"
                            abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "は定義されていません！"
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                        If j < 3 Then
                            For k = j + 1 To 3
                                kb = KPsurf(ib, k)
                                If jb = kb Then
                                    num = num + 1
                                    aa$ = "Error" + LTrim$(Str$(num))
                                    aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                                    abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "が重複しています！"
                                    If num = 10 Then
                                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                        Exit Sub
                                    End If
                                End If
                            Next k
                        End If
                    Next j
                    n1 = KPsurf(ib, 1)
                    n2 = KPsurf(ib, 2)
                    n3 = KPsurf(ib, 3)
                    Call SideSearch(kh, js, n1, n2, n3) '辺の探索
                    If kh = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 辺の指定が間違っています！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    ElseIf kmulti(kh, js) = 1 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 内部辺が指定されています！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    If Pnorm(ib) = 0 And Ptang(ib) = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 荷重がありません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If
        End If

        If Nspring > 0 Then   'ﾊﾞﾈ支持節点のﾁｪｯｸ
            Dim j2 As Integer
            If Ntype = 3 Then
                j2 = 1
            Else
                j2 = 2
            End If
            For ib = 1 To Nspring
                For j = 1 To j2
                    nb = KPspring(ib, j)
                    If j = 1 Then
                        If nb < 1 Or nb > Npoint Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                            abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    ElseIf nb > 0 Then
                        If nb < 1 Or nb > Npoint Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                            abuf$(num) = aa$ + " 節点番号2が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        ElseIf nb = KPspring(ib, 1) Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                            abuf$(num) = aa$ + " 節点番号1、2共" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    End If
                Next j
                ic = 0
                For j = 1 To Ndim
                    If Cspring(ib, j) <> 0 Then
                        ic = 1
                        Exit For
                    End If
                Next j
                If ic = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                    abuf$(num) = aa$ + " バネ定数がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nslope > 0 Then  '傾斜拘束節点ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nslope
                nb = Nsrest(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  傾斜拘束番号" + Str$(ib) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If KindGrav > 0 Then    '自重倍率のﾁｪｯｸ
            If Ntype < 3 Then
                jx = 2
            Else
                jx = 1
            End If
            ic = 0
            For j = 1 To jx
                If GravMulti(j) <> 0 Then
                    ic = 1
                    Exit For
                End If
            Next j
            If ic = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  自重倍率が設定されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, id, j, jb As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "FEM2D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ntype = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nbound = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nforce = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NforceA = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nsurf = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nspring = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nslope = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindGrav = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindRevF = dat(1)
        If KindRevF > 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            RevPS = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            AccGrav = dat(1)
        Else
            RevPS = 0
            AccGrav = 0
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ngaus = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)

        ReDim NNelem(Nelem, 8), NumMat(Nelem)
        ReDim X(Npoint), Y(Npoint)
        ReDim Nrest(Nbound), Jdirect(Nbound, Ndim), Vdirect(Nbound, Ndim)
        ReDim DataMat(Nmat, Mprop)
        ReDim KPforce(Nforce), Force(Nforce, Ndim)
        ReDim ForceA(NforceA, Ndim), XFA(NforceA), YFA(NforceA)
        ReDim KPsurf(Nsurf, 3), Pnorm(Nsurf), Ptang(Nsurf)
        ReDim KPspring(Nspring, 2), Cspring(Nspring, 3), Fspring(Nspring, 3)
        ReDim Nsrest(Nslope), SAngle(Nslope)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 8
                NNelem(i, j) = dat(j + 1)
            Next j
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            X(i) = dat(1)
            Y(i) = dat(2)
        Next i

        aa$ = sr.ReadLine
        If Len(aa$) < 10 Then
            For i = 1 To Nbound
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nrest(i) = dat(3)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 1 To Ndim
                    jb = 4 * (j - 1)
                    Jdirect(i, j) = dat(jb + 1)
                    Vdirect(i, j) = dat(jb + 3)
                Next j
            Next i
        Else
            For i = 1 To Nbound
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nrest(i) = dat(1)
                For j = 1 To Ndim
                    jb = 2 * j
                    Jdirect(i, j) = dat(jb)
                    Vdirect(i, j) = dat(jb + 1)
                Next j
            Next i
        End If

        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To Mprop
                DataMat(i, j) = dat(j)
            Next j
        Next i

        If Nforce > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nforce
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPforce(i) = dat(1)
                For id = 1 To Ndim
                    Force(i, id) = dat(id + 1)
                Next id
            Next i
        End If

        If NforceA > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To NforceA
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                XFA(i) = dat(1)
                YFA(i) = dat(2)
                For id = 1 To Ndim
                    ForceA(i, id) = dat(id + 2)
                Next id
            Next i
        End If

        If Nsurf > 0 Then
            aa$ = sr.ReadLine
            If Ntype = 3 Then
                For i = 1 To Nsurf
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    KPsurf(i, 1) = dat(1)
                    Pnorm(i) = dat(2)
                Next i
            Else
                For i = 1 To Nsurf
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    KPsurf(i, 1) = dat(1)
                    KPsurf(i, 2) = dat(2)
                    KPsurf(i, 3) = dat(3)
                    Pnorm(i) = dat(4)
                    Ptang(i) = dat(5)
                Next i
            End If
        End If

        If Nspring > 0 Then
            aa$ = sr.ReadLine
            If Ntype = 3 Then
                For i = 1 To Nspring
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    KPspring(i, 1) = dat(1)
                    For j = 1 To Ndim
                        jb = 2 * j - 1
                        Cspring(i, j) = dat(jb + 1)
                        Fspring(i, j) = dat(jb + 2)
                    Next j
                Next i
            Else
                For i = 1 To Nspring
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To 2
                        KPspring(i, j) = dat(j)
                    Next j
                    For j = 1 To Ndim
                        jb = 2 * j
                        Cspring(i, j) = dat(jb + 1)
                        Fspring(i, j) = dat(jb + 2)
                    Next j
                Next i
            End If
        End If

        If Nslope > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nslope
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nsrest(i) = dat(1)
                SAngle(i) = dat(2)
            Next i
        End If

        If KindGrav > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            GravMulti(1) = dat(0)
            If Ntype < 3 Then
                GravMulti(2) = dat(1)
            End If
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, id, im, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "FEM2D"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "問題のタイプ" + "," + Str(Ntype)
        sw.WriteLine(aa$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "拘束節点数" + "," + Str(Nbound)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        aa$ = "荷重節点数" + "," + Str(Nforce)
        sw.WriteLine(aa$)
        aa$ = "座標位置荷重数" + ", " + Str(NforceA)
        sw.WriteLine(aa$)
        aa$ = "表面分布荷重数" + "," + Str(Nsurf)
        sw.WriteLine(aa$)
        aa$ = "バネ支持個数" + "," + Str(Nspring)
        sw.WriteLine(aa$)
        aa$ = "傾斜拘束節点数" + "," + Str(Nslope)
        sw.WriteLine(aa$)
        aa$ = "自重の有無（0:なし、1:あり）" + "," + Str(KindGrav)
        sw.WriteLine(aa$)
        aa$ = "回転遠心力の有無（0:なし、1:あり）" + "," + Str(KindRevF)
        sw.WriteLine(aa$)

        If KindRevF > 0 Then
            aa$ = "  回転数（単位時間当り）" + "," + Str(RevPS)
            sw.WriteLine(aa$)
            aa$ = "  重力加速度" + "," + Str(AccGrav)
            sw.WriteLine(aa$)
        End If
        aa$ = "ガウス積分点数" + "," + Str(Ngaus)
        sw.WriteLine(aa$)
        aa$ = "１節点当り変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)
        aa$ = "材料データの成分数" + "," + Str(Mprop)
        sw.WriteLine(aa$)

        aa$ = "要素データ (要素番号、材料番号、8節点番号)"
        sw.WriteLine(aa$)
        For i = 1 To Nelem
            aa$ = Str$(i) + ",   " + Str$(NumMat(i))
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        If Ntype = 4 Then
            aa$ = "節点データ（節点番号、Ｒ座標、Ｚ座標）"
        Else
            aa$ = "節点データ（節点番号、Ｘ座標、Ｙ座標）"
        End If
        sw.WriteLine(aa$)
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            sw.WriteLine(aa$)
        Next i

        aa$ = "拘束節点データ（拘束番号、節点番号、"
        Select Case Ntype
            Case Is < 3
                aa$ = aa$ + "X拘束、δX、Y拘束、δY）"
            Case 3
                aa$ = aa$ + "W拘束、δW、θX拘束、θX、θY拘束、θY）"
            Case 4
                aa$ = aa$ + "R拘束、δR、Z拘束、δZ）"
        End Select
        sw.WriteLine(aa$)
        For i = 1 To Nbound
            aa$ = Str(i) + "," + Str(Nrest(i))
            For j = 1 To Ndim
                aa$ = aa$ + "," + Str(Jdirect(i, j)) + "," + Str(Vdirect(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        aa$ = "材料データ (材料番号、ヤング率、ポアソン比"
        If Ntype < 4 Then
            aa$ = aa$ + "、厚さ"
        End If
        aa$ = aa$ + "、比重量)"
        sw.WriteLine(aa$)
        For im = 1 To Nmat
            aa$ = Str$(im)
            For j = 1 To Mprop
                aa$ = aa$ + ",   " + Str$(DataMat(im, j))
            Next j
            sw.WriteLine(aa$)
        Next im

        If Nforce > 0 Then
            aa$ = "荷重節点 (荷重番号、節点番号､"
            Select Case Ntype
                Case Is < 3
                    aa$ = aa$ + "X方向荷重、Y方向荷重)"
                Case 3
                    aa$ = aa$ + "W方向荷重、θX方向ﾓｰﾒﾝﾄ、θY方向ﾓｰﾒﾝﾄ)"
                Case 4
                    aa$ = aa$ + "R方向荷重、Z方向荷重)"
            End Select
            sw.WriteLine(aa$)
            For i = 1 To Nforce
                aa$ = Str$(i) + ",  " + Str$(KPforce(i))
                For id = 1 To Ndim
                    aa$ = aa$ + ",   " + Str$(Force(i, id))
                Next id
                sw.WriteLine(aa$)
            Next i
        End If

        If NforceA > 0 Then
            aa$ = "座標位置荷重 (荷重番号、"
            Select Case Ntype
                Case Is < 3
                    aa$ = aa$ + "X座標、Y座標、X方向荷重、Y方向荷重)"
                Case 3
                    aa$ = aa$ + "X座標、Y座標、W方向荷重、θX方向ﾓｰﾒﾝﾄ、θY方向ﾓｰﾒﾝﾄ)"
                Case 4
                    aa$ = aa$ + "R座標、Z座標、R方向荷重、Z方向荷重)"
            End Select
            sw.WriteLine(aa$)
            For i = 1 To NforceA
                aa$ = Str$(i) + ",  " + Str$(XFA(i)) + ",  " + Str$(YFA(i))
                For id = 1 To Ndim
                    aa$ = aa$ + ",   " + Str$(ForceA(i, id))
                Next id
                sw.WriteLine(aa$)
            Next i
        End If

        If Nsurf > 0 Then
            If Ntype <> 3 Then
                aa$ = "表面分布荷重（表面荷重番号、３節点番号、"
                aa$ = aa$ + "垂直方向分布荷重、接線方向分布荷重）"
                sw.WriteLine(aa$)
                For i = 1 To Nsurf
                    aa$ = Str$(i)
                    For j = 1 To 3
                        aa$ = aa$ + ",   " + Str$(KPsurf(i, j))
                    Next j
                    aa$ = aa$ + ",   " + Str$(Pnorm(i))
                    aa$ = aa$ + ",   " + Str$(Ptang(i))
                    sw.WriteLine(aa$)
                Next i
            Else
                aa$ = "表面分布荷重（表面荷重番号、要素番号、"
                aa$ = aa$ + "W方向分布荷重）"
                sw.WriteLine(aa$)
                For i = 1 To Nsurf
                    aa$ = Str$(i)
                    aa$ = aa$ + ",   " + Str$(KPsurf(i, 1))
                    aa$ = aa$ + ",   " + Str$(Pnorm(i))
                    sw.WriteLine(aa$)
                Next i
            End If
        End If

        If Nspring > 0 Then
            If Ntype <> 3 Then
                aa$ = "バネ支持データ（No.、節点番号1、節点番号2、"
                If Ntype = 4 Then
                    aa$ = aa$ + "Ｋr、Ｆsr、Ｋz、Ｆsz）"
                Else
                    aa$ = aa$ + "Ｋx、Ｆsx、Ｋy、Ｆsy）"
                End If
                sw.WriteLine(aa$)
                For i = 1 To Nspring
                    aa$ = Str$(i) + ",  " + Str$(KPspring(i, 1)) + ",  " + Str$(KPspring(i, 2))
                    For j = 1 To 2
                        aa$ = aa$ + ",  " + Str$(Cspring(i, j))
                        aa$ = aa$ + ",  " + Str$(Fspring(i, j))
                    Next j
                    sw.WriteLine(aa$)
                Next i
            Else
                aa$ = "バネ支持データ（No.、節点番号、Ｋz、Ｆsz、Ｋxr、Ｆsxr、Ｋyr、Ｆsyr）"
                sw.WriteLine(aa$)
                For i = 1 To Nspring
                    aa$ = Str$(i) + ",  " + Str$(KPspring(i, 1))
                    For j = 1 To 3
                        aa$ = aa$ + ",  " + Str$(Cspring(i, j))
                        aa$ = aa$ + ",  " + Str$(Fspring(i, j))
                    Next j
                    sw.WriteLine(aa$)
                Next i
            End If
        End If

        If Nslope > 0 Then
            aa$ = "傾斜拘束節点データ (拘束番号、節点番号、傾斜角度）"
            sw.WriteLine(aa$)
            For i = 1 To Nslope
                aa$ = Str$(i)
                aa$ = aa$ + ",   " + Str$(Nsrest(i))
                aa$ = aa$ + ",   " + Str$(SAngle(i))
                sw.WriteLine(aa$)
            Next i
        End If

        If KindGrav > 0 Then
            Select Case Ntype
                Case 3
                    sw.WriteLine("自重のW方向の倍率")
                    sw.WriteLine(GravMulti(1))
                Case 4
                    sw.WriteLine("自重のZ方向の倍率")
                    sw.WriteLine(GravMulti(1))
                Case Else
                    sw.WriteLine("自重のX、Y方向の倍率")
                    aa$ = Str$(GravMulti(1)) + ",  " + Str$(GravMulti(2))
                    sw.WriteLine(aa$)
            End Select
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Function DispMax(ByVal k1 As Integer, ByRef k2 As Integer, _
    ByRef tdb(,) As Single) As Single
        '変位の最大値
        'k1,k2･･･要素の開始、終了番号
        'dmax･･･X座標最小値
        'tdb(,)･･･節点の各座標方向変位
        Dim i, ie, j, k As Integer
        Dim b1, dmax As Single
        dmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    For k = 1 To 2
                        b1 = Math.Abs(tdb(i, k))
                        If b1 > dmax Then
                            dmax = b1
                        End If
                    Next k
                Next j
            End If
        Next ie
        Return dmax

    End Function

    Function DispMaxS(ByVal k1 As Integer, ByRef k2 As Integer, _
    ByRef tdb(,) As Single) As Single
        '変位の最大値（板の曲げ）
        'k1,k2･･･要素の開始、終了番号
        'dmax･･･X座標最小値
        'tdb(,)･･･節点の各座標方向変位
        Dim i, ie, j As Integer
        Dim b1, dmax As Single
        dmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    b1 = Math.Abs(tdb(i, 1))
                    If b1 > dmax Then
                        dmax = b1
                    End If
                Next j
            End If
        Next ie
        Return dmax

    End Function

    Sub DispRateSet(ByRef ratio As Single, ByVal tfdb(,) As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'tfdb(,)･･･節点の各座標変位
        Dim b1, rlen, rlen2, xmax, xmin, ymax, ymin As Single
        Dim xbuf(Npoint), ybuf(Npoint) As Single

        Call AxisCheckS(KE1, KE2, xmin, xmax, ymin, ymax, X, Y)
        '座標の最小・最大値のﾁｪｯｸ
        rlen = xmax - xmin
        b1 = ymax - ymin
        If b1 > rlen Then
            rlen = b1
        End If

        rlen2 = DispMax(KE1, KE2, tfdb)
        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DispRateSetS(ByRef ratio As Single, ByVal tfdb(,) As Single)
        '変位作画倍率の設定（板の曲げ）
        'ratio･･･変位作画の倍率
        'tfdb(,)･･･節点の各座標変位
        Dim b1, rlen, rlen2, xmax, xmin, ymax, ymin As Single
        Dim xbuf(Npoint), ybuf(Npoint) As Single

        Call AxisCheckS(KE1, KE2, xmin, xmax, ymin, ymax, X, Y)
        '座標の最小・最大値のﾁｪｯｸ
        rlen = xmax - xmin
        b1 = ymax - ymin
        If b1 > rlen Then
            rlen = b1
        End If

        rlen2 = DispMaxS(KE1, KE2, tfdb)
        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal aa As String, _
        ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)

    End Sub

    Sub DrawScale(ByVal icheck As Integer, ByVal dpixb As Single, _
        ByVal dpiyb As Single, ByVal xlen As Single, ByVal ylen As Single, _
        ByVal xmin As Single, ByVal xmax As Single, _
        ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb
        b1 = xmax - xmin : b2 = ymax - ymin
        Select Case icheck
            Case 0
                If Cfix < 1 Then
                    If b1 <> 0 Then
                        cx = xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2 / Cfix
                    End If
                Else
                    If b1 <> 0 Then
                        cx = Cfix * xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2
                    End If
                End If
                If b1 = 0 Then
                    XYscale = cy
                ElseIf b2 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
            Case 1
                If Cfix < 1 Then
                    If b2 <> 0 Then
                        cx = xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1 / Cfix
                    End If
                Else
                    If b2 <> 0 Then
                        cx = Cfix * xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1
                    End If
                End If
                If b2 = 0 Then
                    XYscale = cy
                ElseIf b1 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
        End Select

    End Sub

    Sub Elem2DPrep()
        '構造図作画準備
        Dim ie As Integer
        Dim pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        KE1 = 1
        KE2 = Nelem
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa, X, Y)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        Call OutSideCheck() '外周辺のﾁｪｯｸ

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemFA(ByRef ke As Integer, ByVal xb As Single, ByVal yb As Single)
        '該当要素の検出
        'ke･･･該当要素
        'xb,yb･･･座標位置
        Dim i, ic, j, jj, k, k1, k2 As Integer
        Dim area, eps, x0, x1, x2, y0, y1, y2 As Single
        Dim xx(3), yy(3) As Single

        ke = 0
        For i = 1 To Nelem
            x0 = 0
            y0 = 0
            For k = 1 To 8
                XE(k) = X(NNelem(i, k))
                YE(k) = Y(NNelem(i, k))
                x0 = x0 + XE(k)
                y0 = y0 + YE(k)
            Next k
            xx(3) = x0 / 8
            yy(3) = y0 / 8
            For k = 1 To 8
                Select Case k
                    Case 1
                        k1 = 1 : k2 = 2
                    Case 2
                        k1 = 2 : k2 = 3
                    Case 3
                        k1 = 3 : k2 = 4
                    Case 4
                        k1 = 4 : k2 = 5
                    Case 5
                        k1 = 5 : k2 = 6
                    Case 6
                        k1 = 6 : k2 = 7
                    Case 7
                        k1 = 7 : k2 = 8
                    Case 8
                        k1 = 8 : k2 = 1
                End Select
                xx(1) = XE(k1) : yy(1) = YE(k1)
                xx(2) = XE(k2) : yy(2) = YE(k2)
                ic = 0
                For j = 1 To 3
                    x1 = xx(j)
                    y1 = yy(j)
                    jj = j + 1
                    If jj > 3 Then
                        jj = 1
                    End If
                    x2 = xx(jj)
                    y2 = yy(jj)
                    Call AreaCalc(area, x1, y1, x2, y2, xb, yb)
                    '三角形の符号付き面積算出（最初の2点に対して左側は正、右側は負）
                    If area < 0 Then
                        ic = -1
                        Exit For
                    End If
                Next j
                If ic = 0 Then
                    ke = i
                    Exit Sub
                End If
            Next k
        Next i

        If ke = 0 Then
            For i = 1 To Nelem
                x0 = 0
                y0 = 0
                For k = 1 To 8
                    XE(k) = X(NNelem(i, k))
                    YE(k) = Y(NNelem(i, k))
                    x0 = x0 + XE(k)
                    y0 = y0 + YE(k)
                Next k
                x0 = x0 / 8
                y0 = y0 / 8
                Call AreaCalc(area, x1, y1, x2, y2, x0, y0) '三角形の符号付き面積算出
                eps = 0.1 * area
                xx(3) = x0
                yy(3) = y0
                ic = 0
                For k = 1 To 8
                    Select Case k
                        Case 1
                            k2 = 1 : k2 = 2
                        Case 2
                            k1 = 2 : k2 = 3
                        Case 3
                            k1 = 3 : k2 = 4
                        Case 4
                            k1 = 4 : k2 = 5
                        Case 5
                            k1 = 5 : k2 = 6
                        Case 6
                            k1 = 6 : k2 = 7
                        Case 7
                            k1 = 7 : k2 = 8
                        Case 8
                            k1 = 8 : k2 = 1
                    End Select
                    xx(1) = XE(k1) : yy(1) = YE(k1)
                    xx(2) = XE(k2) : yy(2) = YE(k2)
                    For j = 1 To 3
                        x1 = xx(j)
                        y1 = yy(j)
                        jj = jj + 1
                        If jj > 3 Then
                            jj = 1
                        End If
                        x2 = xx(jj)
                        y2 = yy(jj)
                        Call AreaCalc(area, x1, y1, x2, y2, xb, yb)
                        '三角形の符号付き面積算出（最初の2点に対して左側は正、右側は負）
                        If area < -eps Then
                            ic = -1
                            Exit For
                        End If
                    Next j
                    If ic = 0 Then
                        ke = i
                        Exit Sub
                    End If
                Next k
            Next i
        End If

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer, _
    ByRef xsb() As Single, ByRef ysb() As Single)
        '拡大作画する要素の探索（回転なし）
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        'xsb(),ysb()･･･節点のｽｸﾘｰﾝ座標
        Dim ic, ie, j, k, kmin, kmax As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    k = NNelem(ie, j)
                    b1 = xsb(k) - Xmouse(1)
                    b2 = xsb(k) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = ysb(k) - Ymouse(2)
                        b2 = ysb(k) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 8 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub ElemSearch2(ByRef k1 As Integer, ByRef k2 As Integer, _
    ByRef xmb() As Single, ByRef ymb() As Single)
        '拡大作画する要素の探索（回転も可）
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        'xmb(),ymb()･･･節点の座標
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    b1 = xmb(ib) - Xmouse(1)
                    b2 = xmb(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        If Kaxis = 0 Then
                            b1 = ymb(ib) - Ymouse(2)
                            b2 = ymb(ib) - Ymouse(1)
                        Else
                            b1 = ymb(ib) - Ymouse(1)
                            b2 = ymb(ib) - Ymouse(2)
                        End If
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 8 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If
    End Sub

    Sub ElemSearch3(ByVal k1 As Integer, ByVal k2 As Integer, ByRef nb As Integer)
        '矩形領域の外周辺の探索
        'k1,k2･･･要素の開始、終了番号
        'nb･･･適合辺の数
        Dim ib, ie, j, jj, k As Integer
        Dim b1, b2 As Single
        Dim kbuf(3) As Integer

        nb = 0
        For ie = 1 To Nelem
            For j = 1 To 4
                KAppSide(ie, j) = 0
            Next j
        Next ie
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    If KEInside(ie, j) = 0 Then
                        jj = 2 * j - 1
                        kbuf(1) = NNelem(ie, jj)
                        kbuf(2) = NNelem(ie, jj + 1)
                        jj = jj + 2
                        If jj > 8 Then
                            jj = 1
                        End If
                        kbuf(3) = NNelem(ie, jj)
                        For k = 1 To 3
                            ib = kbuf(k)
                            b1 = X(ib) - Xmouse(1)
                            b2 = X(ib) - Xmouse(2)
                            If b1 >= 0 And b2 <= 0 Then
                                If Kaxis = 0 Then
                                    b1 = Y(ib) - Ymouse(2)
                                    b2 = Y(ib) - Ymouse(1)
                                Else
                                    b1 = Y(ib) - Ymouse(1)
                                    b2 = Y(ib) - Ymouse(2)
                                End If
                                If b1 >= 0 And b2 <= 0 Then
                                    If k = 3 Then
                                        KAppSide(ie, j) = 1
                                        nb = nb + 1
                                    End If
                                Else
                                    Exit For
                                End If
                            Else
                                Exit For
                            End If
                        Next k
                    End If
                Next j
            End If
        Next ie

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub ForceDupCheck(ByRef nd As Integer)
        '荷重節点重複のﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, id, ii, nb As Integer
        Dim kbuf(Nforce) As Integer

        nd = 0
        For i = 1 To Nforce - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nforce
                    If KPforce(i) = KPforce(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For id = 1 To Ndim
                            If Force(ii, id) > 0 Then
                                Force(i, id) = Force(ii, id)
                            End If
                        Next id
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim kpfbuf(Nforce) As Integer
            Dim forcebuf(Nforce, Ndim) As Single
            For i = 1 To Nforce
                kpfbuf(i) = KPforce(i)
                For id = 1 To Ndim
                    forcebuf(i, id) = Force(i, id)
                Next id
            Next i
            nb = Nforce - nd
            ReDim KPforce(nb), Force(nb, Ndim)
            nb = 0
            For i = 1 To Nforce
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    KPforce(nb) = kpfbuf(i)
                    For id = 1 To Ndim
                        Force(nb, id) = forcebuf(i, id)
                    Next id
                End If
            Next i
            Nforceb = nb
            Nforce = nb
        End If

    End Sub

    Sub HLoadDupCheck(ByRef nd As Integer)
        '分布荷重の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nsurf) As Integer

        nd = 0
        For i = 1 To Nsurf - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nsurf
                    If KPsurf(i, 1) = KPsurf(ii, 1) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        If Ntype = 3 Then
                            Pnorm(i) = Pnorm(ii)
                        Else
                            If Pnorm(ii) <> 0 Then
                                Pnorm(i) = Pnorm(ii)
                            End If
                            If Ptang(ii) <> 0 Then
                                Ptang(i) = Ptang(ii)
                            End If
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim kphbuf(Nsurf, 3) As Integer
            Dim pnbuf(Nsurf), ptbuf(Nsurf) As Single
            For i = 1 To Nsurf
                For j = 1 To 3
                    kphbuf(i, j) = KPsurf(i, j)
                Next j
                pnbuf(i) = Pnorm(i)
                ptbuf(i) = Ptang(i)
            Next i
            nb = Nsurf - nd
            ReDim KPsurf(nb, 3), Pnorm(nb), Ptang(nb)
            nb = 0
            For i = 1 To Nsurf
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 3
                        KPsurf(nb, j) = kphbuf(i, j)
                    Next j
                    Pnorm(nb) = pnbuf(i)
                    Ptang(nb) = ptbuf(i)
                End If
            Next i
            Nsurfb = nb
            Nsurf = nb
        End If

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            If Kaxis = 0 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                yb = (xa - Xlower) / XYscale + YaxisMin
            End If
        Else
            If Kaxis = 0 Then
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale + XaxisMin
                yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
            End If
        End If

    End Sub

    Sub MouseCalc2()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                If Kaxis = 0 Then
                    xb = (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                    yb = (xa - Xlower) / XYscale + YaxisMin
                End If
            Else
                If Kaxis = 0 Then
                    xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale + XaxisMin
                    yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
                End If
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MouseElemSearch(ByVal k1 As Integer, ByVal k2 As Integer, ByRef kx As Integer, _
    ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の要素の探索
        'k1,k2･･･要素の開始、終了番号
        'kx･･･ﾏｳｽ点のある要素番号（0はなし）
        'xb,yb･･･ﾏｳｽ点のﾓﾃﾞﾙ座標位置
        Dim ib, ic, ie, j As Integer
        Dim area, x1, x2, y1, y2 As Single

        kx = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                ic = 0
                ib = NNelem(ie, 8)
                x2 = X(ib) : y2 = Y(ib)
                For j = 1 To 8
                    x1 = x2 : y1 = y2
                    ib = NNelem(ie, j)
                    x2 = X(ib) : y2 = Y(ib)
                    Call AreaCalc(area, x1, y1, x2, y2, xb, yb) '三角形の符号付き面積算出
                    If area < 0 Then
                        ic = -1
                        Exit For
                    End If
                Next j
                If ic = 0 Then
                    kx = ie
                    Exit For
                End If
            End If
        Next ie

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single, _
    ByRef xmb() As Single, ByRef ymb() As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'ky･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        'xmb(),ymb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = xmb(ib) - xb
                        b2 = ymb(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub MousePointSearch2(ByVal k1 As Integer, ByVal k2 As Integer, _
        ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の節点の探索2（板の曲げ）
        'k1,k2･･･要素の開始、終了番号
        'ky･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点のﾓﾃﾞﾙ座標位置
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2, x2, y2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        x2 = Xscreen(ib)
                        y2 = Yscreen(ib)
                        Call XYCalc2(0, x2, y2)     'X,Yの作画座標計算
                        b1 = x2 - xb
                        b2 = y2 - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub NumCheck(ByRef ic As Integer, ByRef i1 As Integer, _
        ByRef i2 As Integer, ByRef j1 As Integer, _
        ByRef j2 As Integer)
        '要素辺の重なりのﾁｪｯｸ
        'ic･･･要素辺の重なり有無（0:なし、1:有り）
        'i1･･･第1の辺の一端の節点番号
        'i2･･･第1の辺の他端の節点番号
        'j1･･･第2の辺の一端の節点番号
        'j2･･･第2の辺の他端の節点番号
        ic = 0
        If i1 = j1 Then
            If i2 = j2 Then
                ic = 1
            End If
        ElseIf i1 = j2 Then
            If i2 = j1 Then
                ic = 1
            End If
        End If

    End Sub

    Sub OutSideCheck()
        '外周辺のﾁｪｯｸ
        Dim i2, ie, ii, j, j2, jj As Integer
        ReDim KEInside(Nelem, 4)

        For ie = 1 To Nelem - 1
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    i2 = NNelem(ie, 2 * j)
                    For ii = ie + 1 To Nelem
                        If KdrawElem(ii) > 0 Then
                            For jj = 1 To 4
                                j2 = NNelem(ii, 2 * jj)
                                If i2 = j2 Then
                                    KEInside(ie, j) = 1
                                    KEInside(ii, jj) = 1
                                End If
                            Next jj
                        End If
                    Next ii
                Next j
            End If
        Next ie

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM2Menu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            For i = 1 To 3
                Koption(i) = Val(sr.ReadLine)
            Next i
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM2Menu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 3
            sw.WriteLine(Koption(i))
        Next i
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByVal rlenscreen As Single, _
    ByRef xvbuf() As Single, ByRef yvbuf() As Single, ByRef zvbuf() As Single, _
    ByRef xsbuf() As Single, ByRef ysbuf() As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        'xvbuf(),yvbuf(),zvbuf()･･･視線座標のX,Y,Z
        'xsbuf(),ysbuf()･･･ｽｸﾘｰﾝ座標のX,Y
        Dim ib, ie, j As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / zvbuf(ib)
                        xsbuf(ib) = b1 * xvbuf(ib)
                        ysbuf(ib) = b1 * (yvbuf(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PointSearch(ByRef nb As Integer)
        '矩形領域の節点の探索
        'nb･･･適合節点の数
        Dim i As Integer
        Dim b1, b2 As Single
        Dim kbuf(3) As Integer

        nb = 0
        For i = 1 To Npoint
            KAppPoint(i) = 0
            b1 = X(i) - Xmouse(1)
            b2 = X(i) - Xmouse(2)
            If b1 >= 0 And b2 <= 0 Then
                If Kaxis = 0 Then
                    b1 = Y(i) - Ymouse(2)
                    b2 = Y(i) - Ymouse(1)
                Else
                    b1 = Y(i) - Ymouse(1)
                    b2 = Y(i) - Ymouse(2)
                End If
                If b1 >= 0 And b2 <= 0 Then
                    KAppPoint(i) = 1
                    nb = nb + 1
                End If
            End If
        Next i

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Private Sub SideSearch(ByRef kh As Integer, ByRef js As Integer,
        ByVal n1 As Integer, ByVal n2 As Integer, ByVal n3 As Integer)
        '辺の探索
        'kh･･･要素番号（0:なし、1:有り）
        'js･･･一致辺の中間節点順序番号
        'n1,n2,n3･･･3節点の番号
        Dim ie, j, jx, k1, k2, k3 As Integer
        kh = 0
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                k1 = NNelem(ie, j)
                If k1 = n1 Then
                    k2 = NNelem(ie, j + 1)
                    If j < 7 Then
                        jx = j + 2
                    Else
                        jx = 1
                    End If
                    k3 = NNelem(ie, jx)
                    If k2 = n2 And k3 = n3 Then
                        kh = ie
                        js = j + 1
                        Exit For
                    End If
                End If
            Next j
            If kh > 0 Then
                Exit For
            End If
        Next ie

    End Sub

    Sub SlopeDupCheck(ByRef nd As Integer)
        '傾斜拘束節点重複のﾁｪｯｸ・修正
        Dim i, ii, nb As Integer
        Dim kbuf(Nslope) As Integer

        nd = 0
        For i = 1 To Nslope - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nslope
                    If Nsrest(i) = Nsrest(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        SAngle(i) = SAngle(ii)
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf(Nslope) As Integer
            Dim cbuf(Nslope) As Single
            For i = 1 To Nslope
                nbuf(i) = Nsrest(i)
                cbuf(i) = SAngle(i)
            Next i
            nb = Nslope - nd
            ReDim Nsrest(nb), SAngle(nb)
            nb = 0
            For i = 1 To Nslope
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    Nsrest(nb) = nbuf(i)
                    SAngle(nb) = cbuf(i)
                End If
            Next i
            Nslopeb = nb
            Nslope = nb
        End If

    End Sub

    Sub SpringCheck2()
        '節点間バネのﾁｪｯｸ
        Dim i, j As Integer
        For i = 1 To Nspring
            If KPspring(i, 2) > 0 Then
                For j = 2 To Ndim
                    Cspring(i, j) = 0
                    Fspring(i, j) = 0
                Next j
            End If
        Next i

    End Sub

    Sub SpringDupCheck(ByRef nd As Integer)
        'ﾊﾞﾈ節点重複のﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nspring) As Integer

        nd = 0
        For i = 1 To Nspring - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nspring
                    If KPspring(i, 1) = KPspring(ii, 1) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Ndim
                            If Cspring(ii, j) <> 0 Then
                                Cspring(i, j) = Cspring(ii, j)
                                Fspring(i, j) = Fspring(ii, j)
                            End If
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim kpbuf(Nspring, 2) As Integer
            Dim csbuf(Nspring, 3), fsbuf(Nspring, 3) As Single
            For i = 1 To Nspring
                For j = 1 To 2
                    kpbuf(i, j) = KPspring(i, j)
                Next j
                For j = 1 To Ndim
                    csbuf(i, j) = Cspring(i, j)
                    fsbuf(i, j) = Fspring(i, j)
                Next j
            Next i
            nb = Nspring - nd
            ReDim KPspring(nb, 2)
            ReDim Cspring(nb, 3), Fspring(nb, 3)
            nb = 0
            For i = 1 To Nspring
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 2
                        KPspring(nb, j) = kpbuf(i, j)
                    Next j
                    For j = 1 To Ndim
                        Cspring(nb, j) = csbuf(i, j)
                        Fspring(nb, j) = fsbuf(i, j)
                    Next j
                End If
            Next i
            Nspringb = nb
            Nspring = nb
        End If

    End Sub

    Sub TimCal(ByVal ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\FEM2D.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\FEM2D.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TrSet()
        'ｶﾞｳｽ点・節点への変換ﾏﾄﾘｯｸｽ設定
        Dim i As Integer
        Dim b1, b2, b3, b4, b5, b6 As Single

        b1 = 1.93694 : b2 = -0.4858871 : b3 = 0.02777768 : b4 = -0.5555556
        b5 = 0.3747762 : b6 = -0.2147176
        For i = 1 To 7 Step 2
            Tr(i, 1) = b3 : Tr(i, 3) = b3 : Tr(i, 7) = b3 : Tr(i, 9) = b3
            Tr(i, 5) = b4
        Next i
        Tr(1, 1) = b1 : Tr(1, 2) = b2 : Tr(1, 4) = b2
        Tr(1, 6) = b5 : Tr(1, 8) = b5 : Tr(1, 9) = b6
        Tr(3, 2) = b5 : Tr(3, 3) = b6 : Tr(3, 4) = b2
        Tr(3, 6) = b5 : Tr(3, 7) = b1 : Tr(3, 8) = b2
        Tr(5, 1) = b6 : Tr(5, 2) = b5 : Tr(5, 4) = b5
        Tr(5, 6) = b2 : Tr(5, 8) = b2 : Tr(5, 9) = b1
        Tr(7, 2) = b2 : Tr(7, 3) = b1 : Tr(7, 4) = b5
        Tr(7, 6) = b2 : Tr(7, 7) = b6 : Tr(7, 8) = b5

        b1 = 0.1666667 : b2 = -0.3333336 : b3 = 0.1666668 : b4 = 1.145497
        b5 = 0 : b6 = -0.1454973
        For i = 2 To 8 Step 2
            Tr(i, 1) = b1 : Tr(i, 3) = b1 : Tr(i, 7) = b1 : Tr(i, 9) = b1
            Tr(i, 5) = b5
        Next i
        Tr(2, 2) = b2 : Tr(2, 3) = b3 : Tr(2, 4) = b4
        Tr(2, 6) = b6 : Tr(2, 8) = b2 : Tr(2, 9) = b3
        Tr(4, 1) = b3 : Tr(4, 2) = b6 : Tr(4, 3) = b3
        Tr(4, 4) = b2 : Tr(4, 6) = b2 : Tr(4, 8) = b4
        Tr(6, 1) = b3 : Tr(6, 2) = b2 : Tr(6, 4) = b6
        Tr(6, 6) = b4 : Tr(6, 7) = b3 : Tr(6, 8) = b2
        Tr(8, 2) = b4 : Tr(8, 4) = b2 : Tr(8, 6) = b2
        Tr(8, 7) = b3 : Tr(8, 8) = b6 : Tr(8, 9) = b3

    End Sub

    Sub UFileAddB()
        'UFileBの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileB(i) = UFileB(0) Then
                For j = i - 1 To 0 Step -1
                    UFileB(j + 1) = UFileB(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileB(i + 1) = UFileB(i)
            Next i
        End If
        UseFileBOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileBIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM2UFileB.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileB(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileBOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM2UFileB.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileB(i) <> "" Then
                sw.WriteLine(UFileB(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

    Sub XYCalc2(ByVal icheck As Integer, ByRef xb As Single, _
        ByRef yb As Single)
        'X,Yの作画座標計算
        'icheck･･･作画時のXY入れ替えの有無（0:なし、1:有り）
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            If icheck = 0 Then
                xb = b1 * XYscale + Xlower
                yb = Yupper - b2 * XYscale * Cfix
            Else
                xb = b2 * XYscale + Xlower
                yb = b1 * XYscale * Cfix + Ylower
            End If
        Else
            If icheck = 0 Then
                xb = b1 * XYscale / Cfix + Xlower
                yb = Yupper - b2 * XYscale
            Else
                xb = b2 * XYscale / Cfix + Xlower
                yb = b1 * XYscale + Ylower
            End If
        End If
    End Sub

End Module
